@@ -13,6 +13,7 @@ devices.
 """
 
 import re
+import datetime
 
 from .util import InvalidMessageError
 
@@ -25,11 +26,14 @@ class BaseMessage(object):
     raw = None
     """The raw message text"""
 
+    timestamp = None
+    """The timestamp of the message"""
+
     def __init__(self):
         """
         Constructor
         """
-        pass
+        self.timestamp = datetime.datetime.now()
 
     def __str__(self):
         """
@@ -37,6 +41,22 @@ class BaseMessage(object):
         """
         return self.raw
 
+    def dict(self, **kwargs):
+        """
+        Dictionary representation.
+        """
+        return dict(
+            time=self.timestamp,
+            mesg=self.raw,
+            **kwargs
+        )
+
+    def __repr__(self):
+        """
+        String representation.
+        """
+        return repr(self.dict())
+
 
 class Message(BaseMessage):
     """
@@ -102,12 +122,6 @@ class Message(BaseMessage):
         if data is not None:
             self._parse_message(data)
 
-    def __str__(self):
-        """
-        String conversion operator.
-        """
-        return self.raw
-
     def _parse_message(self, data):
         """
         Parse the message from the device.
@@ -151,6 +165,37 @@ class Message(BaseMessage):
             # Current cursor location on the alpha display.
             self.cursor_location = int(self.bitfield[21:23], 16)
 
+    def dict(self, **kwargs):
+        """
+        Dictionary representation.
+        """
+        return dict(
+            time                  = self.timestamp,
+            bitfield              = self.bitfield,
+            numeric_code          = self.numeric_code,
+            panel_data            = self.panel_data,
+            mask                  = self.mask,
+            ready                 = self.ready,
+            armed_away            = self.armed_away,
+            armed_home            = self.armed_home,
+            backlight_on          = self.backlight_on,
+            programming_mode      = self.programming_mode,
+            beeps                 = self.beeps,
+            zone_bypassed         = self.zone_bypassed,
+            ac_power              = self.ac_power,
+            chime_on              = self.chime_on,
+            alarm_event_occurred  = self.alarm_event_occurred,
+            alarm_sounding        = self.alarm_sounding,
+            battery_low           = self.battery_low,
+            entry_delay_off       = self.entry_delay_off,
+            fire_alarm            = self.fire_alarm,
+            check_zone            = self.check_zone,
+            perimeter_only        = self.perimeter_only,
+            text                  = self.text,
+            cursor_location       = self.cursor_location,
+            **kwargs
+        )
+
 
 class ExpanderMessage(BaseMessage):
     """
@@ -183,12 +228,6 @@ class ExpanderMessage(BaseMessage):
         if data is not None:
             self._parse_message(data)
 
-    def __str__(self):
-        """
-        String conversion operator.
-        """
-        return self.raw
-
     def _parse_message(self, data):
         """
         Parse the raw message from the device.
@@ -217,6 +256,18 @@ class ExpanderMessage(BaseMessage):
         else:
             raise InvalidMessageError('Unknown expander message header: {0}'.format(data))
 
+    def dict(self, **kwargs):
+        """
+        Dictionary representation.
+        """
+        return dict(
+            time                  = self.timestamp,
+            address               = self.address,
+            channel               = self.channel,
+            value                 = self.value,
+            **kwargs
+        )
+
 
 class RFMessage(BaseMessage):
     """
@@ -246,12 +297,6 @@ class RFMessage(BaseMessage):
         if data is not None:
             self._parse_message(data)
 
-    def __str__(self):
-        """
-        String conversion operator.
-        """
-        return self.raw
-
     def _parse_message(self, data):
         """
         Parses the raw message from the device.
@@ -282,6 +327,19 @@ class RFMessage(BaseMessage):
         except ValueError:
             raise InvalidMessageError('Received invalid message: {0}'.format(data))
 
+    def dict(self, **kwargs):
+        """
+        Dictionary representation.
+        """
+        return dict(
+            time                  = self.timestamp,
+            serial_number         = self.serial_number,
+            value                 = self.value,
+            battery               = self.battery,
+            supervision           = self.supervision,
+            **kwargs
+        )
+
 
 class LRRMessage(BaseMessage):
     """
@@ -307,12 +365,6 @@ class LRRMessage(BaseMessage):
         if data is not None:
             self._parse_message(data)
 
-    def __str__(self):
-        """
-        String conversion operator.
-        """
-        return self.raw
-
     def _parse_message(self, data):
         """
         Parses the raw message from the device.
@@ -330,3 +382,15 @@ class LRRMessage(BaseMessage):
 
         except ValueError:
             raise InvalidMessageError('Received invalid message: {0}'.format(data))
+
+    def dict(self, **kwargs):
+        """
+        Dictionary representation.
+        """
+        return dict(
+            time                  = self.timestamp,
+            event_data            = self.event_data,
+            event_type            = self.event_type,
+            partition             = self.partition,
+            **kwargs
+        )
