@@ -264,3 +264,13 @@ class TestAlarmDecoder(TestCase):
     def test_zonetracker_update(self):
         msg = self._decoder._handle_message('[0000000000000000----],000,[f707000600e5800c0c020000],"                                "')
         self._decoder._zonetracker.update.assert_called_with(msg)
+
+    def test_with(self):
+        self._decoder.close()
+        self._decoder = AlarmDecoder(self._device)
+
+        self.assertIsNotNone(self._decoder._device)
+        with self._decoder.open():
+            pass
+        self.assertIsNone(self._decoder._device)
+
